<?php if (!defined('PmWiki')) exit();
/**
 * txtdb.php
 * @author V.Krishn (vkrishn@insteps.net)
 * @note This file is part of Cookbook recipe "TxtDb" for PmWiki;
 * You can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the  Free Software
 * Foundation; either version 2 of the License, or (at your option)
 * any later version. @see pmwiki.php for full details.
 *
 * @category pmwiki/cookbook
 * @package txtdb
 * @copyright Copyright (c) 2010-2023 V.Krishn (vkrishn@insteps.net)
 * @license GPL
 * @version 2.2.1-alpha-2
 */

global $TxtDb;
if ( ! IsEnabled($EnableTxtDb, 0)) return;

## Current script variables
    if ( ! defined('DS') ) { define('DS', DIRECTORY_SEPARATOR); }
    $TxtDb['Current']['dir'] = dirname(__FILE__);
    $EnableTxtDbDebug = 0; # 0=no debug, 1=debug messages, 2=all
    $TxtDb['debug']['msg'] = array();

## Get cookbook/modules meta and version variables
## @todo xml version of the metapage
    if (file_exists($TxtDb['Current']['dir'] . DS . "version.php")) {
      include_once($TxtDb['Current']['dir'] . DS . "version.php");
    }

## Get wiki group and pagename for use
    $TxtDb['page']['CurrentGroup'] = FmtPageName('$Group', $pagename);
    $TxtDb['page']['CurrentName'] = FmtPageName('$Name', $pagename);

## Set default values for GROUP and Page for Txtdb
    SDV($TxtDb['default']['DbGroupReserved'], 'TxtDb'); #not implemented
    SDV($TxtDb['default']['DbPageGroup'], 'Content');
    SDV($TxtDb['default']['DbPageName'], 'TxtDb');

// ========================================================
 /*
   Make appropriate settings with the default password = dbpass
     in txtdb.php or in config.php if commenting below.
   Read main documentation on page protection and passwords.
   Simple security precaution below
 */
  global $DefaultPasswords;
  $tmpLen = strlen($TxtDb['default']['DbPageName']);
  if ($TxtDb['default']['DbPageName'] === substr($pagename, -$tmpLen)) {
    $cfunc = $VersionNum >= 2002090 ? 'pmcrypt' : 'crypt'; # >= 2002075
    $DefaultPasswords['edit'] = $cfunc('menupass');
    $DefaultPasswords['attr'] = $cfunc('menupass');
  }
// ========================================================

## Set modules/cookbook install variables.
## Change these values accordingly.
    SDV($TxtDb['Install']['location'], '0');        # 0=field, 1=farm, 2=others
    SDV($TxtDb['Install']['path'], 'cookbook');     # relative to field or farm
    SDV($TxtDb['Install']['dir'], 'txtdb');         # folder where installed
    SDV($TxtDb['Install']['icon']['dir'], 'icons'); # icons to be used in dbs #todo


    // --------------------------------------
    // Would not require to edit below this
    // --------------------------------------



// //////////////////////////////////////////////////////////
//(:txtdbSet id wikipage dbname:)
//    id - unique id per db call
//    wikipage - txtdb wikipage name
//    dbname - txtdb name
// //////////////////////////////////////////////////////////

# improves full page edit performance, but preview would not be available
if ($action == 'edit') { return; }

# NOTE Solutions for preg related callback functions with /e modifier in php 5.5x.
  # Using function Markup_e for compat solution
  # for new format, no /e , min-requirement: pmwiki-2.2.58+ and php 5.5x.
  # else ...
  /* use old format, for pmwiki < 2.2.58
     if using php 5.5x see pmwiki documentation as how to suppress
       deprecated preg_replace errors. */
if(function_exists('Markup_e')) {

    Markup('txtdbSet','directives','/\\(:txtdbSet\\s(.*?):\\)/',"CallbackTxtDb");
    Markup('txtdbQuery','>txtdbSet','/\\(:txtdbQuery\\s(.*?):\\)/',"CallbackTxtDb");
    Markup('txtdbDraw','>txtdbQuery','/\\(:txtdbDraw\\s(.*?):\\)/',"CallbackTxtDb"); #todo
    Markup('txtdbDrawVar','<{$var}','/\\(:txtdbDrawVar\\s(.*?):\\)/',"CallbackTxtDb"); #todo

    Markup('txtdbPageVar', '<txtdbDrawVar',
      "/\\(:txtdbPageVar(\\s+([^\n]*?))?:\\)[^\\S\n]*\n(.*?)\\(:txtdbPageVarEnd:\\)/si", "CallbackTxtDb");

} else {

    Markup('txtdbSet','directives','/\\(:txtdbSet\\s(.*?):\\)/e',"txtdbSet('$1')");
    Markup('txtdbQuery','>txtdbSet','/\\(:txtdbQuery\\s(.*?):\\)/e',"txtdbQuery(PSS('$1'))");
    Markup('txtdbDraw','>txtdbQuery','/\\(:txtdbDraw\\s(.*?):\\)/e',"'<:block>'.txtdbDraw('$1')"); #todo
    Markup('txtdbDrawVar','<{$var}','/\\(:txtdbDrawVar\\s(.*?):\\)/e',"txtdbDraw('$1')"); #todo

    Markup('txtdbPageVar', '<txtdbDrawVar',
      "/\\(:txtdbPageVar(\\s+([^\n]*?))?:\\)[^\\S\n]*\n(.*?)\\(:txtdbPageVarEnd:\\)/sei",
      "txtdbPageVar(\$pagename, PSS('$3'), PSS('$1'))");

}

function CallbackTxtDb($m) {
    $a = array('/^\(/', '/\)$/', '/^\s/', '/\s$/');
    $str = explode(' ', trim(preg_replace($a, '', $m[0]), ':'));
    $str1 = explode("\n", $str[0]);
    $str2 = explode(' ', trim(preg_replace($a, '', $str1[0]), ':'));

    switch (trim($str2[0])) {
        case "txtdbSet":
            return txtdbSet($m[1]);
            break;
        case "txtdbQuery":
            return txtdbQuery(PSS($m[1]));
            break;
        case "txtdbDraw":
            return '<:block>'.txtdbDraw($m[1]);
            break;
        case "txtdbDrawVar":
            return txtdbDraw($m[1]);
            break;
        case "txtdbPageVar":
             extract($GLOBALS["MarkupToHTML"]); # to get $pagename
             return txtdbPageVar($pagename, PSS($m[3]), PSS($m[1]));
            break;
    }

}

//global $txtdbDSN, $TxtDb, $TDB;

//Handle form posting
if(isset($_POST['txtdbFrmId']) && isset($_POST['txtdbSubmit'])) { #reserved id for form posting # TODO
    //print '<pre>'; print_r($_POST); print '</pre>'; exit;
    extract($_POST);
    $dbParams = $txtdbFrmId . ' ' . $dbpage . ' ' . $dbname;
    $dbQueryParams = $txtdbFrmId . ' "' . $query . '"';
    txtdbSet($dbParams);
    txtdbQuery($dbQueryParams);
    //print '<pre>'; print_r($TxtDb); print '</pre>'; exit;
}

function txtdbPageVar($pagename, $text, $opt = '') {
    global $TxtDb;
    //$opt = ParseArgs($opt); # TODO
    $html = MarkupToHTML($pagename, $text, array('escape' => 0));
    return keep($html);
}

function txtdbSet($params) {
    global $TxtDb;
    $GLOBALS['CSCHAR'] = '#';

    $args = ParseArgs($params);
    $count = count($args[""]);
    $id = $args[''][0];

    $TxtDb[$id]['dbpage'] = $args[''][1];
    $TxtDb[$id]['dbname'] = $args[''][2];
}

function txtdbQuery($params) {
    global $txtdbDSN, $TxtDb;
    include_once('php-txt-db-api/txt-db-api.php');
    include_once('scripts/functions.php');

    $args = ParseArgs($params);
    $count = count($args[""]);
    $id = $args[''][0];

    $query = $TxtDb[$id]['query'] = $args[''][1];
    $query = stripslashes($query);

    $dbpage = $TxtDb[$id]['dbpage'];
    $dbname = $TxtDb[$id]['dbname'];

    $TDB = new Database($dbpage , $dbname);
    $TxtDb[$id]['result'] = $TDB->executeQuery($query);
}



function txtdbDraw($params) { #todo
  global $txtdbDSN, $TxtDb;

  static $drawcount; $drawcount++;
  $drawparams = array();
  include_once('scripts/functions.php');
  $args = ParseArgs($params);
  $count = isset($args['']) ? count($args[""]) : 0; // issue PHP 7.2
  $DB_CALL_ID = $id = $args[''][0];
  if(isset($TxtDb[$DB_CALL_ID]['drawcount'])) { $TxtDb[$DB_CALL_ID]['drawcount']++;}
     else { $TxtDb[$DB_CALL_ID]['drawcount'] = 1; }

  $TxtDb[$id]['drawparams'] = $args;
  $drawparams =& $TxtDb[$id]['drawparams'];
  $result =& $TxtDb[$id]['result'];

    $data = array();
    if( array_key_exists('', $drawparams) && count($drawparams['']) > 1) {
      $outbehaviour = isset($drawparams[''][2]) ? explode(',', strtolower($drawparams[''][2])) : array(); }

    if(isset($result->rows) && @count($result->rows) > 0) {
      if(in_array('transpose', $outbehaviour)) {
        $data[] = $result->getColumnNames();
      }
      while($result->next()) {
        //echo "Current Row: ";
        //$data = $result->getCurrentValuesAsHash();
        $data[]=$result->getCurrentValues();
      }
      $result->reset();
      //print '<pre>'; print_r($data); print '</pre>';
    } else {
        if( strtolower(@$drawparams[''][1]) !== 'sessionvar') return;
    }


    if(strpos(strtolower(@$drawparams[''][1]), 'table') !== false) {
      include($TxtDb['Current']['dir'] . DS . 'scripts'. DS .'output.table.php');
    }
    if(strpos(strtolower(@$drawparams[''][1]), 'var') !== false) {
      include($TxtDb['Current']['dir'] . DS . 'scripts'. DS .'output.pagevar.php');
    }
    if(strpos(strtolower(@$drawparams[''][1]), 'list') !== false) {
      include($TxtDb['Current']['dir'] . DS . 'scripts'. DS .'output.list.php');
    }
    if(
       strpos(strtolower(@$drawparams[''][1]), 'array') !== false ||
       strpos(strtolower(@$drawparams[''][1]), 'serialize') !== false
      ) {
      include($TxtDb['Current']['dir'] . DS . 'scripts'. DS .'output.raw.php');
    }


    if(@$TxtDb[$DB_CALL_ID]['drawparams']['isRecallable'] === '0' ) {
      unset($result);
      unset($TxtDb[$DB_CALL_ID]['result']); //free result
    }
    //print '<pre>'; print_r(@$txtdbOutFmt); print '</pre>'; exit;
    //return '<pre>'.keep(print_r($_SESSION, true)).'</pre>';

    return @$txtdbOutFmt;

}



